import { getNodeList } from './nodelist';

function appendImage(target, base64) {
  return new Promise((resolve) => {
    const image = document.createElement('img');
    image.className = 'cw-preview-image';

    image.onload = () => {
      target.appendChild(image);
      resolve('image appended');
    };

    image.onerror = () => {
      resolve('failed to upload image to target');
    };

    image.src = base64;
  });
}

/**
 * Generates an image and injects it into the specified target
 * @param {Object} opts - The parameters required to execute the preview function
 * @param {string|Element|NodeList} [opts.target] - The target element the image will be appended into
 * @return {Promise} A promise - The resolved promise will have a response argument containing the result of the request
 */
function preview(opts) {
  return new Promise((resolve, reject) => {
    const mandatory = ['target', 'image'];
    const properties = Object.keys(opts);
    const error = mandatory.reduce((fieldsMissing, property) => {
      if (properties.indexOf(property) === -1) {
        return `${fieldsMissing}, ${property}`;
      }
      return fieldsMissing;
    }, '');

    if (error) {
      window.console.error(`Missing mandatory properties: ${error.replace(', ', '')}`);
      reject(`Missing mandatory properties: ${error.replace(', ', '')}`);
      return;
    }

    Promise.all(getNodeList(opts.target).map((item) => {
      return appendImage(item, opts.image);
    })).then(() => {
      resolve('image pushed to preview target(s)');
    });
  });
}

export {
  preview
};
