import { uploadImage } from './components/image-upload';
import { startCamera, takePicture } from './components/video-capture';
import { resize, crop } from './components/image-transform';
import { preview } from './components/image-render';

function updateContext(context, params) {
  const parsed = Object.keys(params).reduce((data, key) => {
    if (key !== 'target') {
      Object.assign(context, { [key]: params[key] });
    }
    Object.assign(data, { [key]: params[key] });
    return data;
  }, Object.assign({}, context || {}));

  if (!parsed.target) {
    return new Error('You are missing a target element!');
  }

  return parsed;
}

/**
 * @function
 * Initialises the image handler
 * @param {string|Element|NodeList} target - The element to will be the target of all subsequent actions
 * @returns {Object} A new instance of the image handler
 */
class ImageHandler {
  constructor(target = 'body') {
    this.target = target;
  }

  crop(opts = {}) {
    const parsed = updateContext(this, opts);
    return parsed instanceof Error ? Promise.reject(parsed) : crop(parsed).then(data => Promise.resolve(updateContext(this, data)));
  }
  resize(opts = {}) {
    const parsed = updateContext(this, opts);
    return parsed instanceof Error ? Promise.reject(parsed) : resize(parsed).then(data => Promise.resolve(updateContext(this, data)));
  }
  preview(opts = {}) {
    const parsed = updateContext(this, opts);
    return parsed instanceof Error ? Promise.reject(parsed) : preview(parsed).then(data => Promise.resolve(updateContext(this, data)));
  }
  uploadImage(opts = {}) {
    const parsed = updateContext(this, opts);
    return parsed instanceof Error ? Promise.reject(parsed) : uploadImage(parsed).then(data => Promise.resolve(updateContext(this, data)));
  }
  startCamera(opts = {}) {
    const parsed = updateContext(this, opts);
    return parsed instanceof Error ? Promise.reject(parsed) : startCamera(parsed).then(data => Promise.resolve(updateContext(this, data)));
  }
  takePicture(opts = {}) {
    const parsed = updateContext(this, opts);
    return parsed instanceof Error ? Promise.reject(parsed) : takePicture(parsed).then(data => Promise.resolve(updateContext(this, data)));
  }
}

// function ImageHandler() {
//   return {
//     crop,
//     resize,
//     preview,
//     uploadImage,
//     startCamera,
//     takePicture
//   };
// }
export { ImageHandler };
